/**
 * Un example d'utilisation de GtkImage pour tracer les deux
 * fonctions mathematiques Y=cos(X) et Y=sin(X)/2
 * sur l'intervalle X dans [-6.0 ; +6.0]
 * et avec Y dans [-0.5 ; +1.0]
 *
 * An example using a GtkImage in order to draw the two
 * maths fonctions Y=cos(X) et Y=sin(X)/2
 * with X ranging in [-6.0 ; +6.0] 
 * and Y in [-0.5 ; +1.0]
 *
 * AUTHOR: Guillaume Riviere (C) 2015
 * FILE: main.c
 */

#include <math.h>
#include <gtk/gtk.h>

#include "gdkpixbuf_accessors.h"

/* Callbacks pour le signal de destruction de la fenetre principale */
G_MODULE_EXPORT void on_mainwindow_destroy (GtkObject *object, gpointer user_data) {
  gtk_main_quit () ;  /* Stopper la boucle GTK */
}

int X_toScreen (double Xval, double Xmin, double Xmax, int width) {
  return (((Xval - Xmin) * width) / (Xmax - Xmin)) ;
}

int Y_toScreen (double Yval, double Ymin, double Ymax, int height) {
  return (((Yval - Ymin) * height) / (Ymax - Ymin)) ;
}

double X_fromScreen (int xScreen, double Xmin, double Xmax, int width) {
  return Xmin + ((xScreen * (Xmax - Xmin)) / width) ;
}

double Y_fromScreen (int yScreen, double Ymin, double Ymax, int height) {
  return Ymin + ((yScreen * (Ymax - Ymin)) / height) ;
}


/* Fonction principale */
int main (int argc, char *argv[]) {
  GtkBuilder *builder = NULL ;
  GtkWindow  *mainwindow = NULL ;
  GtkImage   *image1 = NULL ;
  GdkPixbuf  *gdk_pixbuf = NULL ;
  int w, h ;
  int x, y ;
  double X, Y ;

  gtk_init (&argc, &argv);

  /* Creer tous les widgets grace au builder GTK */
  builder = gtk_builder_new ();
  gtk_builder_add_from_file (builder, "gtk_image_example_2.24.glade", NULL);
  gtk_builder_connect_signals (builder, NULL);

  /*------------*/

  mainwindow = GTK_WINDOW (gtk_builder_get_object (builder, "mainwindow"));
  image1 = GTK_IMAGE (gtk_builder_get_object (builder, "image1"));

  /* Creer un GdkPixbuf depuis un fichier PNG */
  gdk_pixbuf = gdk_pixbuf_new_from_file ("gtk.png", NULL);
  w = gdk_pixbuf_get_width (gdk_pixbuf) ;
  h = gdk_pixbuf_get_height (gdk_pixbuf) ;

  /* Tracer axe X */
  y = Y_toScreen (0, -0.5, 1.0, h) ;
  for (x=0 ; x<w ; x++) {
    gdkpixbuf_set_pixel_color (gdk_pixbuf, x, h-y, 0, 0, 0) ;
  }

  /* Tracer axe Y */
  x = X_toScreen (0, -6.0, 6.0, h) ;
  for (y=0 ; y<h ; y++) {
    gdkpixbuf_set_pixel_color (gdk_pixbuf, x, h-y, 0, 0, 0) ;
  }

  /* Tracer la fonction Y=cos(X) */  
  for (X=-6.0 ;  X<=6.0 ; X+=0.005) {

    Y = cos (X) ;

    x = X_toScreen (X, -6.0, 6.0, w) ;
    y = Y_toScreen (Y, -0.5, 1.0, h) ;

    gdkpixbuf_set_pixel_color (gdk_pixbuf, x, h-y, 255, 0, 0) ;
  }

  /* Tracer la fonction Y=sin(X)/2 */  
  for (X=-6.0 ;  X<=6.0 ; X+=0.005) {

    Y = sin (X) * 0.5 ;

    x = X_toScreen (X, -6.0, 6.0, w) ;
    y = Y_toScreen (Y, -0.5, 1.0, h) ;

    gdkpixbuf_set_pixel_color (gdk_pixbuf, x, h-y, 0, 255, 0) ;
  }
  
  /* Afficher le pixbuf dans un GtkImage */
  gtk_image_set_from_pixbuf (image1, gdk_pixbuf) ;
  gtk_widget_queue_draw(GTK_WIDGET(image1));

  /*------------*/

  gtk_widget_show_all (GTK_WIDGET(mainwindow)); /* ATTENTION */

  /* Lancer la boucle GTK */
  gtk_main ();

  /* La boucle GTK est terminee, detruire maintenant tous les widgets */
  g_object_unref (G_OBJECT (builder));

  /* Fin normale du programme */
  return 0;
}

